package specs

import ocs "github.com/opencontainers/runtime-spec/specs-go"

type (
	// ProcessSpec aliases the platform process specs
	ProcessSpec ocs.Process
	// Spec aliases the platform oci spec
	Spec ocs.Spec
)
